import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * A simple program to demonstrate data races
 *  
 * Author: John C. Linford (jlinford@vt.edu)
 * Date: 25 August 2005
 * License: Public Domain
 */
public class DataRaceApplet extends JApplet {

	private int[] shared = new int[1];
	private Thread t1, t2;
	
	private JTextArea outputArea = new JTextArea();
	private JScrollPane outputScroller = new JScrollPane(outputArea);
	private JButton runBtn = new JButton("Race!");
	
	
	public DataRaceApplet() {
		setLayout(new BorderLayout());
		this.add(runBtn, BorderLayout.SOUTH);
		this.add(outputScroller, BorderLayout.CENTER);
		
		runBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				race();
			}
		});
	}
	
	
	private void race() {
		
        outputArea.setText("");
		
        // Create a first thread to increment shared
        t1 = new Thread() {
            public void run() {
                int copy = 0; // Stores a copy of shared to detect races

                while(t1 != null) {
                    // Make a copy of shared in a safe way
                    synchronized(shared) {
                        copy = shared[0];
                    }

                    // Increment shared unsafely (possible race)
                    shared[0] = shared[0] + 1;

                    // Check for a race condition in a safe way
                    synchronized(shared) {
                        outputArea.append("[t1] -- Shared: " + shared[0] + ".\n");
                        if(shared[0] - 1 != copy) {
                            outputArea.append("[t1] -- Data Race!  Had " + shared[0] + " when I expected " + (copy + 1) + ".\n");
                            t1 = t2 = null;
                        }
                    }
                }
            }
        };
        
        // Create a second thread to increment shared
        t2 = new Thread() {
            public void run() {
                int copy = 0; // Stores a copy of shared to detect races

                while(t2 != null) {
                    // Make a copy of shared in a safe way
                    synchronized(shared) {
                        copy = shared[0];
                    }

                    // Increment shared unsafely (possible race)
                    shared[0] = shared[0] + 1;

                    // Check for a race condition in a safe way
                    synchronized(shared) {
                    	outputArea.append("[t2] -- Shared: " + shared[0] + ".\n");
                        if(shared[0] - 1 != copy) {
                        	outputArea.append("[t2] -- Data Race!  Had " + shared[0] + " when I expected " + (copy + 1) + ".\n");
                            t1 = t2 = null;
                        }
                    }
                }
            }
        };
        
        t1.start();
        t2.start();
	}
}
